@@ -5,6 +5,7 @@ import grpc
 import logging
 from contextlib import asynccontextmanager
 import itertools
+from urllib.parse import urlparse
 
 import HStream.Server.HStreamApi_pb2 as ApiPb
 import HStream.Server.HStreamApi_pb2_grpc as ApiGrpc
@@ -30,7 +31,7 @@ from hstreamdb.utils import (
     encode_records_from_append_payload,
 )
 
-__all__ = ["insecure_client", "HStreamDBClient"]
+__all__ = ["secure_client", "insecure_client", "HStreamDBClient"]
 
 logger = logging.getLogger(__name__)
 
@@ -66,15 +67,21 @@ class HStreamDBClient:
 
     _cons_target = staticmethod(lambda host, port: f"{host}:{port}")
 
-    def __init__(self, host: str = "127.0.0.1", port: int = 6570):
+    def __init__(
+        self, host: str = "127.0.0.1", port: int = 6570, credentials=None
+    ):
         self._current_target = self._cons_target(host, port)
         self._channels = {}
         self._append_channels = {}
         self._subscription_channels = {}
         self._reader_channels = {}
         self._shards_info = {}
-        # TODO: secure_channel
-        _channel = grpc.aio.insecure_channel(self._current_target)
+        if credentials:
+            _channel = grpc.aio.secure_channel(
+                self._current_target, credentials
+            )
+        else:
+            _channel = grpc.aio.insecure_channel(self._current_target)
         self._channels[self._current_target] = _channel
         self._stub = ApiGrpc.HStreamApiStub(_channel)
 
@@ -223,9 +230,10 @@ class HStreamDBClient:
         )
         return r.exists
 
-    @dec_api
     async def delete_subscription(self, subscription_id: str, force=False):
-        await self._stub.DeleteSubscription(
+        channel = await self._lookup_subscription(subscription_id)
+        stub = ApiGrpc.HStreamApiStub(channel)
+        await stub.DeleteSubscription(
             ApiPb.DeleteSubscriptionRequest(
                 subscriptionId=subscription_id, force=force
             )
@@ -300,8 +308,7 @@ class HStreamDBClient:
     async def read_reader(
         self, reader_id: str, max_records: str
     ) -> Iterator[Record]:
-        channel = await self._lookup_reader(reader_id)
-        stub = ApiGrpc.HStreamApiStub(channel)
+        stub = await self._lookup_reader_stub(reader_id)
         resp = await stub.ReadShard(
             ApiPb.ReadShardRequest(readerId=reader_id, maxRecords=max_records)
         )
@@ -310,9 +317,9 @@ class HStreamDBClient:
             decode_records(r) for r in resp.receivedRecords
         )
 
-    @dec_api
     async def delete_reader(self, reader_id: str) -> None:
-        await self._stub.DeleteShardReader(
+        stub = await self._lookup_reader_stub(reader_id)
+        await stub.DeleteShardReader(
             ApiPb.DeleteShardReaderRequest(readerId=reader_id)
         )
         return None
@@ -373,6 +380,10 @@ class HStreamDBClient:
 
         return keyid, self._get_channel(target)
 
+    async def _lookup_append_stub(self, name, key, shard_id):
+        keyid, channel = self._lookup_append(name, key, shard_id)
+        return keyid, ApiGrpc.HStreamApiStub(channel)
+
     async def _lookup_subscription(self, subscription_id: str):
         target = self._subscription_channels.get(subscription_id)
         if not target:
@@ -386,6 +397,10 @@ class HStreamDBClient:
 
         return self._get_channel(target)
 
+    async def _lookup_subscription_stub(self, subscription_id: str):
+        channel = self._lookup_subscription(subscription_id)
+        return ApiGrpc.HStreamApiStub(channel)
+
     async def _lookup_reader(self, reader_id: str):
         target = self._reader_channels.get(reader_id)
         if not target:
@@ -397,6 +412,10 @@ class HStreamDBClient:
 
         return self._get_channel(target)
 
+    async def _lookup_reader_stub(self, reader_id: str):
+        channel = await self._lookup_reader(reader_id)
+        return ApiGrpc.HStreamApiStub(channel)
+
     @dec_api
     async def _lookup_append_api(self, shard_id):
         r = await self._stub.LookupShard(
@@ -470,16 +489,81 @@ class HStreamDBClient:
                 await channel.close(grace=None)
 
 
-async def insecure_client(host="127.0.0.1", port=6570):
+async def insecure_client(host="127.0.0.1", port=6570, url=None):
     """Creates an insecure client to a cluster.
 
     Args:
         host: hostname to connect to HStreamDB, defaults to '127.0.0.1'
         port: port to connect to HStreanDB, defaults to 6570
+        url: alternative service url to connect to HStreamDB, it should be in
+            'hstream://your-host' format. Note that if you provide this url
+            then the 'host' and 'port' args will be ignored.
 
     Returns:
         A :class:`HStreamDBClient`
     """
+    if url:
+        o = urlparse(url)
+        if o.scheme != "hstream":
+            raise KeyError(f"Invalid service url scheme {o.scheme}")
+        host = o.hostname
+        # FIXME: should the default port be the same as secure_client?
+        port = o.port or 6570
     client = HStreamDBClient(host=host, port=port)
     await client.init_cluster_info()
     return client
+
+
+async def secure_client(
+    host="127.0.0.1",
+    port=6570,
+    url=None,
+    is_creds_file=False,
+    root_certificates=None,
+    private_key=None,
+    certificate_chain=None,
+):
+    """Creates a secure client to a cluster.
+
+    Args:
+        host: hostname to connect to HStreamDB, defaults to '127.0.0.1'
+        port: port to connect to HStreanDB, defaults to 6570
+        url: alternative service url to connect to HStreamDB, it should be in
+            'hstreams://your-host' format. Note that if you provide this url
+            then the 'host' and 'port' args will be ignored.
+        is_creds_file: whether the credentials is a filepath or the contents.
+        root_certificates: The PEM-encoded root certificates as a byte string,
+            or None to retrieve them from a default location chosen by gRPC
+            runtime. Note: if 'is_creds_file' is True this is the filepath
+            instead of the contents.
+        private_key: The PEM-encoded private key as a byte string, or None if no
+            private key should be used. Note: if 'is_creds_file' is True this
+            is the filepath instead of the contents.
+        certificate_chain: The PEM-encoded certificate chain as a byte string
+            to use or None if no certificate chain should be used. Note: if
+            'is_creds_file' is True this is the filepath instead of the contents.
+
+    Returns:
+        A :class:`HStreamDBClient`
+    """
+    if is_creds_file:
+        with open(root_certificates, "rb") as f_cert, open(
+            private_key, "rb"
+        ) as f_key, open(certificate_chain, "rb") as f_chain:
+            root_certificates = f_cert.read()
+            private_key = f_key.read()
+            certificate_chain = f_chain.read()
+    creds = grpc.ssl_channel_credentials(
+        root_certificates=root_certificates,
+        private_key=private_key,
+        certificate_chain=certificate_chain,
+    )
+    if url:
+        o = urlparse(url)
+        if o.scheme != "hstreams":
+            raise KeyError(f"Invalid service url scheme {o.scheme}")
+        host = o.hostname
+        port = o.port or 6570
+    client = HStreamDBClient(host=host, port=port, credentials=creds)
+    await client.init_cluster_info()
+    return client
